/* TimedQueue.c generated by valac 0.10.3, the Vala compiler
 * generated from TimedQueue.vala, do not modify */

/* Copyright 2010 Yorba Foundation
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution. 
 */
/* TimedQueue is a specialized collection class.  It holds items in order, but rather than being*/
/* manually dequeued, they are dequeued automatically after a specified amount of time has elapsed*/
/* for that item.  As of today, it's possible the item will be dequeued a bit later than asked*/
/* for, but it will never be early.  Future implementations might tighten up the lateness.*/
/**/
/* The original design was to use a signal to notify when an item has been dequeued, but Vala has*/
/* a bug with passing an unnamed type as a signal parameter:*/
/* https://bugzilla.gnome.org/show_bug.cgi?id=628639*/
/**/
/* The rate the items come off the queue can be spaced out.  Note that this can cause items to back*/
/* up.  As of today, TimedQueue makes no effort to combat this.*/

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <time.h>
#include <gobject/gvaluecollector.h>


#define TYPE_TIMED_QUEUE (timed_queue_get_type ())
#define TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TIMED_QUEUE, TimedQueue))
#define TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TIMED_QUEUE, TimedQueueClass))
#define IS_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TIMED_QUEUE))
#define IS_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TIMED_QUEUE))
#define TIMED_QUEUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TIMED_QUEUE, TimedQueueClass))

typedef struct _TimedQueue TimedQueue;
typedef struct _TimedQueueClass TimedQueueClass;
typedef struct _TimedQueuePrivate TimedQueuePrivate;

#define TYPE_SORTED_LIST (sorted_list_get_type ())
#define SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_LIST, SortedList))
#define SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_LIST, SortedListClass))
#define IS_SORTED_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_LIST))
#define IS_SORTED_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_LIST))
#define SORTED_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_LIST, SortedListClass))

typedef struct _SortedList SortedList;
typedef struct _SortedListClass SortedListClass;

#define TIMED_QUEUE_TYPE_ELEMENT (timed_queue_element_get_type ())
#define TIMED_QUEUE_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElement))
#define TIMED_QUEUE_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))
#define TIMED_QUEUE_IS_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_IS_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))

typedef struct _TimedQueueElement TimedQueueElement;
typedef struct _TimedQueueElementClass TimedQueueElementClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _TimedQueueElementPrivate TimedQueueElementPrivate;
#define _timed_queue_element_unref0(var) ((var == NULL) ? NULL : (var = (timed_queue_element_unref (var), NULL)))
#define _g_destroy_func0(var) (((var == NULL) || (g_destroy_func == NULL)) ? NULL : (var = (g_destroy_func (var), NULL)))
typedef struct _TimedQueueParamSpecElement TimedQueueParamSpecElement;
typedef struct _ParamSpecTimedQueue ParamSpecTimedQueue;

#define TYPE_HASH_TIMED_QUEUE (hash_timed_queue_get_type ())
#define HASH_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_TIMED_QUEUE, HashTimedQueue))
#define HASH_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_TIMED_QUEUE, HashTimedQueueClass))
#define IS_HASH_TIMED_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_TIMED_QUEUE))
#define IS_HASH_TIMED_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_TIMED_QUEUE))
#define HASH_TIMED_QUEUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_TIMED_QUEUE, HashTimedQueueClass))

typedef struct _HashTimedQueue HashTimedQueue;
typedef struct _HashTimedQueueClass HashTimedQueueClass;
typedef struct _HashTimedQueuePrivate HashTimedQueuePrivate;

typedef void (*DequeuedCallback) (gconstpointer item, void* user_data);
struct _TimedQueue {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TimedQueuePrivate * priv;
};

struct _TimedQueueClass {
	GTypeClass parent_class;
	void (*finalize) (TimedQueue *self);
	void (*notify_dequeued) (TimedQueue* self, gconstpointer item);
	void (*clear) (TimedQueue* self);
	gboolean (*contains) (TimedQueue* self, gconstpointer item);
	gboolean (*enqueue) (TimedQueue* self, gconstpointer item);
	gboolean (*enqueue_many) (TimedQueue* self, GeeCollection* items);
	gboolean (*remove_first) (TimedQueue* self, gconstpointer item);
	void (*paused) (TimedQueue* self, gboolean is_paused);
	gint (*get_size) (TimedQueue* self);
};

struct _TimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	guint hold_msec;
	DequeuedCallback callback;
	gpointer callback_target;
	GDestroyNotify callback_target_destroy_notify;
	GEqualFunc equal_func;
	gint priority;
	guint timer_id;
	SortedList* queue;
	guint dequeue_spacing_msec;
	time_t last_dequeue;
	gboolean paused_state;
};

typedef gint64 (*Comparator) (void* a, void* b, void* user_data);
struct _TimedQueueElement {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TimedQueueElementPrivate * priv;
	gpointer item;
	time_t ready;
};

struct _TimedQueueElementClass {
	GTypeClass parent_class;
	void (*finalize) (TimedQueueElement *self);
};

struct _TimedQueueElementPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _TimedQueueParamSpecElement {
	GParamSpec parent_instance;
};

struct _ParamSpecTimedQueue {
	GParamSpec parent_instance;
};

struct _HashTimedQueue {
	TimedQueue parent_instance;
	HashTimedQueuePrivate * priv;
};

struct _HashTimedQueueClass {
	TimedQueueClass parent_class;
};

struct _HashTimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeHashMap* item_count;
};


static gpointer timed_queue_element_parent_class = NULL;
static gpointer timed_queue_parent_class = NULL;
static gpointer hash_timed_queue_parent_class = NULL;

gpointer timed_queue_ref (gpointer instance);
void timed_queue_unref (gpointer instance);
GParamSpec* param_spec_timed_queue (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_timed_queue (GValue* value, gpointer v_object);
void value_take_timed_queue (GValue* value, gpointer v_object);
gpointer value_get_timed_queue (const GValue* value);
GType timed_queue_get_type (void) G_GNUC_CONST;
GType sorted_list_get_type (void) G_GNUC_CONST;
static gpointer timed_queue_element_ref (gpointer instance);
static void timed_queue_element_unref (gpointer instance);
static GParamSpec* timed_queue_param_spec_element (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) G_GNUC_UNUSED;
static void timed_queue_value_set_element (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static void timed_queue_value_take_element (GValue* value, gpointer v_object) G_GNUC_UNUSED;
static gpointer timed_queue_value_get_element (const GValue* value) G_GNUC_UNUSED;
static GType timed_queue_element_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define TIMED_QUEUE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TIMED_QUEUE, TimedQueuePrivate))
enum  {
	TIMED_QUEUE_DUMMY_PROPERTY
};
TimedQueue* timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority);
TimedQueue* timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority);
static gint64 timed_queue_element_comparator (void* a, void* b);
static gint64 _timed_queue_element_comparator_comparator (void* a, void* b, gpointer self);
SortedList* sorted_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator cmp, void* cmp_target);
SortedList* sorted_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, Comparator cmp, void* cmp_target);
static guint timed_queue_get_heartbeat_timeout (TimedQueue* self);
static gboolean timed_queue_on_heartbeat (TimedQueue* self);
static gboolean _timed_queue_on_heartbeat_gsource_func (gpointer self);
guint timed_queue_get_dequeue_spacing_msec (TimedQueue* self);
void timed_queue_set_dequeue_spacing_msec (TimedQueue* self, guint msec);
void timed_queue_notify_dequeued (TimedQueue* self, gconstpointer item);
static void timed_queue_real_notify_dequeued (TimedQueue* self, gconstpointer item);
gboolean timed_queue_is_paused (TimedQueue* self);
void timed_queue_pause (TimedQueue* self);
void timed_queue_unpause (TimedQueue* self);
void timed_queue_clear (TimedQueue* self);
static void timed_queue_real_clear (TimedQueue* self);
gboolean timed_queue_contains (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_contains (TimedQueue* self, gconstpointer item);
gboolean timed_queue_enqueue (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_enqueue (TimedQueue* self, gconstpointer item);
static time_t timed_queue_calc_ready_time (TimedQueue* self);
static TimedQueueElement* timed_queue_element_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, time_t ready);
static TimedQueueElement* timed_queue_element_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, time_t ready);
gboolean timed_queue_enqueue_many (TimedQueue* self, GeeCollection* items);
static gboolean timed_queue_real_enqueue_many (TimedQueue* self, GeeCollection* items);
gboolean sorted_list_add_list (SortedList* self, GeeList* items);
gboolean timed_queue_remove_first (TimedQueue* self, gconstpointer item);
static gboolean timed_queue_real_remove_first (TimedQueue* self, gconstpointer item);
gulong now_ms (void);
gpointer sorted_list_get_at (SortedList* self, gint index);
gpointer sorted_list_remove_at (SortedList* self, gint index);
gint timed_queue_get_size (TimedQueue* self);
void timed_queue_paused (TimedQueue* self, gboolean is_paused);
static void timed_queue_real_paused (TimedQueue* self, gboolean is_paused);
#define TIMED_QUEUE_ELEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementPrivate))
enum  {
	TIMED_QUEUE_ELEMENT_DUMMY_PROPERTY
};
static void timed_queue_element_finalize (TimedQueueElement* obj);
static void timed_queue_finalize (TimedQueue* obj);
GType hash_timed_queue_get_type (void) G_GNUC_CONST;
#define HASH_TIMED_QUEUE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_HASH_TIMED_QUEUE, HashTimedQueuePrivate))
enum  {
	HASH_TIMED_QUEUE_DUMMY_PROPERTY
};
HashTimedQueue* hash_timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority);
HashTimedQueue* hash_timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority);
static void hash_timed_queue_real_notify_dequeued (TimedQueue* base, gconstpointer item);
static void hash_timed_queue_removed (HashTimedQueue* self, gconstpointer item);
static void hash_timed_queue_real_clear (TimedQueue* base);
static gboolean hash_timed_queue_real_contains (TimedQueue* base, gconstpointer item);
static gboolean hash_timed_queue_real_enqueue (TimedQueue* base, gconstpointer item);
static gboolean hash_timed_queue_real_enqueue_many (TimedQueue* base, GeeCollection* items);
static gboolean hash_timed_queue_real_remove_first (TimedQueue* base, gconstpointer item);
static void hash_timed_queue_finalize (TimedQueue* obj);



#line 31 "TimedQueue.vala"
static gint64 _timed_queue_element_comparator_comparator (void* a, void* b, gpointer self) {
#line 244 "TimedQueue.c"
	gint64 result;
	result = timed_queue_element_comparator (a, b);
	return result;
}


#line 168 "TimedQueue.vala"
static gboolean _timed_queue_on_heartbeat_gsource_func (gpointer self) {
#line 253 "TimedQueue.c"
	gboolean result;
	result = timed_queue_on_heartbeat (self);
	return result;
}


#line 52 "TimedQueue.vala"
TimedQueue* timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority) {
#line 262 "TimedQueue.c"
	TimedQueue* self = (TimedQueue*) g_type_create_instance (object_type);
	DequeuedCallback _tmp0_;
	GEqualFunc _tmp1_ = NULL;
	SortedList* _tmp2_;
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
#line 54 "TimedQueue.vala"
	self->priv->hold_msec = hold_msec;
#line 55 "TimedQueue.vala"
	self->priv->callback = (_tmp0_ = callback, ((self->priv->callback_target_destroy_notify == NULL) ? NULL : (self->priv->callback_target_destroy_notify (self->priv->callback_target), NULL), self->priv->callback = NULL, self->priv->callback_target = NULL, self->priv->callback_target_destroy_notify = NULL), self->priv->callback_target = callback_target, self->priv->callback_target_destroy_notify = NULL, _tmp0_);
#line 56 "TimedQueue.vala"
	if (equal_func != NULL) {
#line 56 "TimedQueue.vala"
		_tmp1_ = equal_func;
#line 278 "TimedQueue.c"
	} else {
#line 56 "TimedQueue.vala"
		_tmp1_ = gee_functions_get_equal_func_for (g_type);
#line 282 "TimedQueue.c"
	}
#line 56 "TimedQueue.vala"
	self->priv->equal_func = _tmp1_;
#line 57 "TimedQueue.vala"
	self->priv->priority = priority;
#line 59 "TimedQueue.vala"
	self->priv->queue = (_tmp2_ = sorted_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, timed_queue_element_unref, _timed_queue_element_comparator_comparator, NULL), _g_object_unref0 (self->priv->queue), _tmp2_);
#line 61 "TimedQueue.vala"
	self->priv->timer_id = g_timeout_add_full (priority, timed_queue_get_heartbeat_timeout (self), _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
#line 292 "TimedQueue.c"
	return self;
}


#line 52 "TimedQueue.vala"
TimedQueue* timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GEqualFunc equal_func, gint priority) {
#line 52 "TimedQueue.vala"
	return timed_queue_construct (TYPE_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, equal_func, priority);
#line 301 "TimedQueue.c"
}


#line 69 "TimedQueue.vala"
guint timed_queue_get_dequeue_spacing_msec (TimedQueue* self) {
#line 307 "TimedQueue.c"
	guint result = 0U;
#line 69 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0U);
#line 311 "TimedQueue.c"
	result = self->priv->dequeue_spacing_msec;
#line 70 "TimedQueue.vala"
	return result;
#line 315 "TimedQueue.c"
}


#line 73 "TimedQueue.vala"
void timed_queue_set_dequeue_spacing_msec (TimedQueue* self, guint msec) {
#line 73 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 74 "TimedQueue.vala"
	if (msec == self->priv->dequeue_spacing_msec) {
#line 75 "TimedQueue.vala"
		return;
#line 327 "TimedQueue.c"
	}
#line 77 "TimedQueue.vala"
	if (self->priv->timer_id != 0) {
#line 78 "TimedQueue.vala"
		g_source_remove (self->priv->timer_id);
#line 333 "TimedQueue.c"
	}
#line 80 "TimedQueue.vala"
	self->priv->dequeue_spacing_msec = msec;
#line 82 "TimedQueue.vala"
	self->priv->timer_id = g_timeout_add_full (self->priv->priority, timed_queue_get_heartbeat_timeout (self), _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
#line 339 "TimedQueue.c"
}


#line 85 "TimedQueue.vala"
static guint timed_queue_get_heartbeat_timeout (TimedQueue* self) {
#line 345 "TimedQueue.c"
	guint result = 0U;
	guint _tmp0_ = 0U;
#line 85 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0U);
#line 86 "TimedQueue.vala"
	if (self->priv->dequeue_spacing_msec == 0) {
#line 87 "TimedQueue.vala"
		_tmp0_ = self->priv->hold_msec / 10;
#line 354 "TimedQueue.c"
	} else {
#line 88 "TimedQueue.vala"
		_tmp0_ = self->priv->dequeue_spacing_msec / 2;
#line 358 "TimedQueue.c"
	}
	result = CLAMP (_tmp0_, (guint) 10, G_MAXUINT);
#line 86 "TimedQueue.vala"
	return result;
#line 363 "TimedQueue.c"
}


#line 91 "TimedQueue.vala"
static void timed_queue_real_notify_dequeued (TimedQueue* self, gconstpointer item) {
#line 91 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 92 "TimedQueue.vala"
	self->priv->callback (item, self->priv->callback_target);
#line 373 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
void timed_queue_notify_dequeued (TimedQueue* self, gconstpointer item) {
#line 52 "TimedQueue.vala"
	TIMED_QUEUE_GET_CLASS (self)->notify_dequeued (self, item);
#line 381 "TimedQueue.c"
}


#line 95 "TimedQueue.vala"
gboolean timed_queue_is_paused (TimedQueue* self) {
#line 387 "TimedQueue.c"
	gboolean result = FALSE;
#line 95 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 391 "TimedQueue.c"
	result = self->priv->paused_state;
#line 96 "TimedQueue.vala"
	return result;
#line 395 "TimedQueue.c"
}


#line 99 "TimedQueue.vala"
void timed_queue_pause (TimedQueue* self) {
#line 99 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 100 "TimedQueue.vala"
	if (self->priv->paused_state) {
#line 101 "TimedQueue.vala"
		return;
#line 407 "TimedQueue.c"
	}
#line 103 "TimedQueue.vala"
	self->priv->paused_state = TRUE;
#line 105 "TimedQueue.vala"
	g_signal_emit_by_name (self, "paused", TRUE);
#line 413 "TimedQueue.c"
}


#line 108 "TimedQueue.vala"
void timed_queue_unpause (TimedQueue* self) {
#line 108 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 109 "TimedQueue.vala"
	if (!self->priv->paused_state) {
#line 110 "TimedQueue.vala"
		return;
#line 425 "TimedQueue.c"
	}
#line 112 "TimedQueue.vala"
	self->priv->paused_state = FALSE;
#line 114 "TimedQueue.vala"
	g_signal_emit_by_name (self, "paused", FALSE);
#line 431 "TimedQueue.c"
}


#line 117 "TimedQueue.vala"
static void timed_queue_real_clear (TimedQueue* self) {
#line 117 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 118 "TimedQueue.vala"
	gee_collection_clear (GEE_COLLECTION (self->priv->queue));
#line 441 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
void timed_queue_clear (TimedQueue* self) {
#line 52 "TimedQueue.vala"
	TIMED_QUEUE_GET_CLASS (self)->clear (self);
#line 449 "TimedQueue.c"
}


#line 121 "TimedQueue.vala"
static gboolean timed_queue_real_contains (TimedQueue* self, gconstpointer item) {
#line 455 "TimedQueue.c"
	gboolean result = FALSE;
#line 121 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 459 "TimedQueue.c"
	{
		GeeIterator* _e_it;
#line 122 "TimedQueue.vala"
		_e_it = gee_iterable_iterator (GEE_ITERABLE (self->priv->queue));
#line 122 "TimedQueue.vala"
		while (TRUE) {
#line 466 "TimedQueue.c"
			TimedQueueElement* e;
#line 122 "TimedQueue.vala"
			if (!gee_iterator_next (_e_it)) {
#line 122 "TimedQueue.vala"
				break;
#line 472 "TimedQueue.c"
			}
#line 122 "TimedQueue.vala"
			e = (TimedQueueElement*) gee_iterator_get (_e_it);
#line 123 "TimedQueue.vala"
			if (self->priv->equal_func (item, e->item)) {
#line 478 "TimedQueue.c"
				result = TRUE;
				_timed_queue_element_unref0 (e);
				_g_object_unref0 (_e_it);
#line 124 "TimedQueue.vala"
				return result;
#line 484 "TimedQueue.c"
			}
			_timed_queue_element_unref0 (e);
		}
		_g_object_unref0 (_e_it);
	}
	result = FALSE;
#line 127 "TimedQueue.vala"
	return result;
#line 493 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
gboolean timed_queue_contains (TimedQueue* self, gconstpointer item) {
#line 52 "TimedQueue.vala"
	return TIMED_QUEUE_GET_CLASS (self)->contains (self, item);
#line 501 "TimedQueue.c"
}


#line 130 "TimedQueue.vala"
static gboolean timed_queue_real_enqueue (TimedQueue* self, gconstpointer item) {
#line 507 "TimedQueue.c"
	gboolean result = FALSE;
	TimedQueueElement* _tmp0_;
	gboolean _tmp1_;
#line 130 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 513 "TimedQueue.c"
	result = (_tmp1_ = gee_collection_add (GEE_COLLECTION (self->priv->queue), _tmp0_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, item, timed_queue_calc_ready_time (self))), _timed_queue_element_unref0 (_tmp0_), _tmp1_);
#line 131 "TimedQueue.vala"
	return result;
#line 517 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
gboolean timed_queue_enqueue (TimedQueue* self, gconstpointer item) {
#line 52 "TimedQueue.vala"
	return TIMED_QUEUE_GET_CLASS (self)->enqueue (self, item);
#line 525 "TimedQueue.c"
}


#line 134 "TimedQueue.vala"
static gboolean timed_queue_real_enqueue_many (TimedQueue* self, GeeCollection* items) {
#line 531 "TimedQueue.c"
	gboolean result = FALSE;
	time_t ready_time;
	GeeArrayList* elements;
#line 134 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 134 "TimedQueue.vala"
	g_return_val_if_fail (GEE_IS_COLLECTION (items), FALSE);
#line 135 "TimedQueue.vala"
	ready_time = timed_queue_calc_ready_time (self);
#line 137 "TimedQueue.vala"
	elements = gee_array_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, timed_queue_element_unref, NULL);
#line 543 "TimedQueue.c"
	{
		GeeIterator* _item_it;
#line 138 "TimedQueue.vala"
		_item_it = gee_iterable_iterator (GEE_ITERABLE (items));
#line 138 "TimedQueue.vala"
		while (TRUE) {
#line 550 "TimedQueue.c"
			gpointer item;
			TimedQueueElement* _tmp0_;
#line 138 "TimedQueue.vala"
			if (!gee_iterator_next (_item_it)) {
#line 138 "TimedQueue.vala"
				break;
#line 557 "TimedQueue.c"
			}
#line 138 "TimedQueue.vala"
			item = gee_iterator_get (_item_it);
#line 139 "TimedQueue.vala"
			gee_abstract_collection_add (GEE_ABSTRACT_COLLECTION (elements), _tmp0_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, item, ready_time));
#line 563 "TimedQueue.c"
			_timed_queue_element_unref0 (_tmp0_);
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		_g_object_unref0 (_item_it);
	}
	result = sorted_list_add_list (self->priv->queue, GEE_LIST (elements));
	_g_object_unref0 (elements);
#line 141 "TimedQueue.vala"
	return result;
#line 573 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
gboolean timed_queue_enqueue_many (TimedQueue* self, GeeCollection* items) {
#line 52 "TimedQueue.vala"
	return TIMED_QUEUE_GET_CLASS (self)->enqueue_many (self, items);
#line 581 "TimedQueue.c"
}


#line 144 "TimedQueue.vala"
static gboolean timed_queue_real_remove_first (TimedQueue* self, gconstpointer item) {
#line 587 "TimedQueue.c"
	gboolean result = FALSE;
	GeeIterator* iter;
#line 144 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 145 "TimedQueue.vala"
	iter = gee_iterable_iterator (GEE_ITERABLE (self->priv->queue));
#line 146 "TimedQueue.vala"
	while (TRUE) {
#line 596 "TimedQueue.c"
		TimedQueueElement* e;
#line 146 "TimedQueue.vala"
		if (!gee_iterator_next (iter)) {
#line 146 "TimedQueue.vala"
			break;
#line 602 "TimedQueue.c"
		}
#line 147 "TimedQueue.vala"
		e = (TimedQueueElement*) gee_iterator_get (iter);
#line 148 "TimedQueue.vala"
		if (self->priv->equal_func (item, e->item)) {
#line 149 "TimedQueue.vala"
			gee_iterator_remove (iter);
#line 610 "TimedQueue.c"
			result = TRUE;
			_timed_queue_element_unref0 (e);
			_g_object_unref0 (iter);
#line 151 "TimedQueue.vala"
			return result;
#line 616 "TimedQueue.c"
		}
		_timed_queue_element_unref0 (e);
	}
	result = FALSE;
	_g_object_unref0 (iter);
#line 155 "TimedQueue.vala"
	return result;
#line 624 "TimedQueue.c"
}


#line 52 "TimedQueue.vala"
gboolean timed_queue_remove_first (TimedQueue* self, gconstpointer item) {
#line 52 "TimedQueue.vala"
	return TIMED_QUEUE_GET_CLASS (self)->remove_first (self, item);
#line 632 "TimedQueue.c"
}


#line 164 "TimedQueue.vala"
static time_t timed_queue_calc_ready_time (TimedQueue* self) {
#line 638 "TimedQueue.c"
	time_t result = 0;
#line 164 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), 0);
#line 642 "TimedQueue.c"
	result = (time_t) (now_ms () + self->priv->hold_msec);
#line 165 "TimedQueue.vala"
	return result;
#line 646 "TimedQueue.c"
}


#line 168 "TimedQueue.vala"
static gboolean timed_queue_on_heartbeat (TimedQueue* self) {
#line 652 "TimedQueue.c"
	gboolean result = FALSE;
	time_t now;
#line 168 "TimedQueue.vala"
	g_return_val_if_fail (IS_TIMED_QUEUE (self), FALSE);
#line 169 "TimedQueue.vala"
	if (self->priv->paused_state) {
#line 659 "TimedQueue.c"
		result = TRUE;
#line 170 "TimedQueue.vala"
		return result;
#line 663 "TimedQueue.c"
	}
#line 172 "TimedQueue.vala"
	now = (time_t) 0;
#line 667 "TimedQueue.c"
	{
		gboolean _tmp0_;
#line 174 "TimedQueue.vala"
		_tmp0_ = TRUE;
#line 174 "TimedQueue.vala"
		while (TRUE) {
#line 674 "TimedQueue.c"
			TimedQueueElement* head;
			gboolean _tmp1_ = FALSE;
			TimedQueueElement* h;
#line 174 "TimedQueue.vala"
			if (!_tmp0_) {
#line 680 "TimedQueue.c"
			}
#line 174 "TimedQueue.vala"
			_tmp0_ = FALSE;
#line 175 "TimedQueue.vala"
			if (gee_collection_get_size (GEE_COLLECTION (self->priv->queue)) == 0) {
#line 176 "TimedQueue.vala"
				break;
#line 688 "TimedQueue.c"
			}
#line 178 "TimedQueue.vala"
			head = (TimedQueueElement*) sorted_list_get_at (self->priv->queue, 0);
#line 179 "TimedQueue.vala"
			g_assert (head != NULL);
#line 181 "TimedQueue.vala"
			if (now == 0) {
#line 182 "TimedQueue.vala"
				now = (time_t) now_ms ();
#line 698 "TimedQueue.c"
			}
#line 184 "TimedQueue.vala"
			if (head->ready > now) {
#line 702 "TimedQueue.c"
				_timed_queue_element_unref0 (head);
#line 185 "TimedQueue.vala"
				break;
#line 706 "TimedQueue.c"
			}
#line 188 "TimedQueue.vala"
			if (self->priv->dequeue_spacing_msec != 0) {
#line 188 "TimedQueue.vala"
				_tmp1_ = (now - self->priv->last_dequeue) < self->priv->dequeue_spacing_msec;
#line 712 "TimedQueue.c"
			} else {
#line 188 "TimedQueue.vala"
				_tmp1_ = FALSE;
#line 716 "TimedQueue.c"
			}
#line 188 "TimedQueue.vala"
			if (_tmp1_) {
#line 720 "TimedQueue.c"
				_timed_queue_element_unref0 (head);
#line 189 "TimedQueue.vala"
				break;
#line 724 "TimedQueue.c"
			}
#line 191 "TimedQueue.vala"
			h = (TimedQueueElement*) sorted_list_remove_at (self->priv->queue, 0);
#line 192 "TimedQueue.vala"
			g_assert (head == h);
#line 194 "TimedQueue.vala"
			timed_queue_notify_dequeued (self, head->item);
#line 195 "TimedQueue.vala"
			self->priv->last_dequeue = now;
#line 199 "TimedQueue.vala"
			if (self->priv->dequeue_spacing_msec != 0) {
#line 736 "TimedQueue.c"
				_timed_queue_element_unref0 (h);
				_timed_queue_element_unref0 (head);
#line 200 "TimedQueue.vala"
				break;
#line 741 "TimedQueue.c"
			}
			_timed_queue_element_unref0 (h);
			_timed_queue_element_unref0 (head);
		}
	}
	result = TRUE;
#line 203 "TimedQueue.vala"
	return result;
#line 750 "TimedQueue.c"
}


gint timed_queue_get_size (TimedQueue* self) {
	return TIMED_QUEUE_GET_CLASS (self)->get_size (self);
}


static gint timed_queue_real_get_size (TimedQueue* base) {
	gint result;
	TimedQueue* self;
	self = base;
	result = gee_collection_get_size (GEE_COLLECTION (self->priv->queue));
#line 160 "TimedQueue.vala"
	return result;
#line 766 "TimedQueue.c"
}


#line 46 "TimedQueue.vala"
static void timed_queue_real_paused (TimedQueue* self, gboolean is_paused) {
#line 46 "TimedQueue.vala"
	g_return_if_fail (IS_TIMED_QUEUE (self));
#line 774 "TimedQueue.c"
}


#line 26 "TimedQueue.vala"
static TimedQueueElement* timed_queue_element_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, time_t ready) {
#line 780 "TimedQueue.c"
	TimedQueueElement* self = (TimedQueueElement*) g_type_create_instance (object_type);
	gconstpointer _tmp0_;
	gpointer _tmp1_;
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
#line 27 "TimedQueue.vala"
	self->item = (_tmp1_ = (_tmp0_ = item, ((_tmp0_ == NULL) || (g_dup_func == NULL)) ? ((gpointer) _tmp0_) : g_dup_func ((gpointer) _tmp0_)), _g_destroy_func0 (self->item), _tmp1_);
#line 28 "TimedQueue.vala"
	self->ready = ready;
#line 791 "TimedQueue.c"
	return self;
}


#line 26 "TimedQueue.vala"
static TimedQueueElement* timed_queue_element_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, gconstpointer item, time_t ready) {
#line 26 "TimedQueue.vala"
	return timed_queue_element_construct (TIMED_QUEUE_TYPE_ELEMENT, g_type, g_dup_func, g_destroy_func, item, ready);
#line 800 "TimedQueue.c"
}


#line 31 "TimedQueue.vala"
static gint64 timed_queue_element_comparator (void* a, void* b) {
#line 806 "TimedQueue.c"
	gint64 result = 0LL;
	result = ((gint64) TIMED_QUEUE_ELEMENT ((TimedQueueElement*) a)->ready) - ((gint64) TIMED_QUEUE_ELEMENT ((TimedQueueElement*) b)->ready);
#line 32 "TimedQueue.vala"
	return result;
#line 811 "TimedQueue.c"
}


static void timed_queue_value_element_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void timed_queue_value_element_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		timed_queue_element_unref (value->data[0].v_pointer);
	}
}


static void timed_queue_value_element_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = timed_queue_element_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer timed_queue_value_element_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* timed_queue_value_element_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		TimedQueueElement* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = timed_queue_element_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* timed_queue_value_element_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	TimedQueueElement** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = timed_queue_element_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* timed_queue_param_spec_element (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	TimedQueueParamSpecElement* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TIMED_QUEUE_TYPE_ELEMENT), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer timed_queue_value_get_element (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT), NULL);
	return value->data[0].v_pointer;
}


static void timed_queue_value_set_element (GValue* value, gpointer v_object) {
	TimedQueueElement* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		timed_queue_element_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_element_unref (old);
	}
}


static void timed_queue_value_take_element (GValue* value, gpointer v_object) {
	TimedQueueElement* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_element_unref (old);
	}
}


static void timed_queue_element_class_init (TimedQueueElementClass * klass) {
	timed_queue_element_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_ELEMENT_CLASS (klass)->finalize = timed_queue_element_finalize;
	g_type_class_add_private (klass, sizeof (TimedQueueElementPrivate));
}


static void timed_queue_element_instance_init (TimedQueueElement * self) {
	self->priv = TIMED_QUEUE_ELEMENT_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void timed_queue_element_finalize (TimedQueueElement* obj) {
	TimedQueueElement * self;
	self = TIMED_QUEUE_ELEMENT (obj);
	((self->item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->item = (self->priv->g_destroy_func (self->item), NULL));
}


static GType timed_queue_element_get_type (void) {
	static volatile gsize timed_queue_element_type_id__volatile = 0;
	if (g_once_init_enter (&timed_queue_element_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { timed_queue_value_element_init, timed_queue_value_element_free_value, timed_queue_value_element_copy_value, timed_queue_value_element_peek_pointer, "p", timed_queue_value_element_collect_value, "p", timed_queue_value_element_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (TimedQueueElementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_element_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueueElement), 0, (GInstanceInitFunc) timed_queue_element_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType timed_queue_element_type_id;
		timed_queue_element_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueueElement", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&timed_queue_element_type_id__volatile, timed_queue_element_type_id);
	}
	return timed_queue_element_type_id__volatile;
}


static gpointer timed_queue_element_ref (gpointer instance) {
	TimedQueueElement* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void timed_queue_element_unref (gpointer instance) {
	TimedQueueElement* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TIMED_QUEUE_ELEMENT_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void value_timed_queue_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_timed_queue_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		timed_queue_unref (value->data[0].v_pointer);
	}
}


static void value_timed_queue_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = timed_queue_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_timed_queue_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_timed_queue_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		TimedQueue* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = timed_queue_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_timed_queue_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	TimedQueue** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = timed_queue_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_timed_queue (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecTimedQueue* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_TIMED_QUEUE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_timed_queue (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE), NULL);
	return value->data[0].v_pointer;
}


void value_set_timed_queue (GValue* value, gpointer v_object) {
	TimedQueue* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		timed_queue_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_unref (old);
	}
}


void value_take_timed_queue (GValue* value, gpointer v_object) {
	TimedQueue* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		timed_queue_unref (old);
	}
}


static void timed_queue_class_init (TimedQueueClass * klass) {
	timed_queue_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_CLASS (klass)->finalize = timed_queue_finalize;
	g_type_class_add_private (klass, sizeof (TimedQueuePrivate));
	TIMED_QUEUE_CLASS (klass)->notify_dequeued = timed_queue_real_notify_dequeued;
	TIMED_QUEUE_CLASS (klass)->clear = timed_queue_real_clear;
	TIMED_QUEUE_CLASS (klass)->contains = timed_queue_real_contains;
	TIMED_QUEUE_CLASS (klass)->enqueue = timed_queue_real_enqueue;
	TIMED_QUEUE_CLASS (klass)->enqueue_many = timed_queue_real_enqueue_many;
	TIMED_QUEUE_CLASS (klass)->remove_first = timed_queue_real_remove_first;
	TIMED_QUEUE_CLASS (klass)->paused = timed_queue_real_paused;
	TIMED_QUEUE_CLASS (klass)->get_size = timed_queue_real_get_size;
	g_signal_new ("paused", TYPE_TIMED_QUEUE, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (TimedQueueClass, paused), NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
}


static void timed_queue_instance_init (TimedQueue * self) {
	self->priv = TIMED_QUEUE_GET_PRIVATE (self);
	self->priv->timer_id = (guint) 0;
	self->priv->dequeue_spacing_msec = (guint) 0;
	self->priv->last_dequeue = (time_t) 0;
	self->priv->paused_state = FALSE;
	self->ref_count = 1;
}


static void timed_queue_finalize (TimedQueue* obj) {
	TimedQueue * self;
	self = TIMED_QUEUE (obj);
	{
#line 65 "TimedQueue.vala"
		if (self->priv->timer_id != 0) {
#line 66 "TimedQueue.vala"
			g_source_remove (self->priv->timer_id);
#line 1121 "TimedQueue.c"
		}
	}
	(self->priv->callback_target_destroy_notify == NULL) ? NULL : (self->priv->callback_target_destroy_notify (self->priv->callback_target), NULL);
	self->priv->callback = NULL;
	self->priv->callback_target = NULL;
	self->priv->callback_target_destroy_notify = NULL;
	_g_object_unref0 (self->priv->queue);
}


GType timed_queue_get_type (void) {
	static volatile gsize timed_queue_type_id__volatile = 0;
	if (g_once_init_enter (&timed_queue_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { value_timed_queue_init, value_timed_queue_free_value, value_timed_queue_copy_value, value_timed_queue_peek_pointer, "p", value_timed_queue_collect_value, "p", value_timed_queue_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (TimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueue), 0, (GInstanceInitFunc) timed_queue_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType timed_queue_type_id;
		timed_queue_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueue", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&timed_queue_type_id__volatile, timed_queue_type_id);
	}
	return timed_queue_type_id__volatile;
}


gpointer timed_queue_ref (gpointer instance) {
	TimedQueue* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void timed_queue_unref (gpointer instance) {
	TimedQueue* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TIMED_QUEUE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 212 "TimedQueue.vala"
HashTimedQueue* hash_timed_queue_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority) {
#line 1166 "TimedQueue.c"
	HashTimedQueue* self = NULL;
	GeeHashMap* _tmp0_;
#line 214 "TimedQueue.vala"
	self = (HashTimedQueue*) timed_queue_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, hold_msec, callback, callback_target, equal_func, priority);
#line 214 "TimedQueue.vala"
	self->priv->g_type = g_type;
#line 214 "TimedQueue.vala"
	self->priv->g_dup_func = g_dup_func;
#line 214 "TimedQueue.vala"
	self->priv->g_destroy_func = g_destroy_func;
#line 216 "TimedQueue.vala"
	self->priv->item_count = (_tmp0_ = gee_hash_map_new (g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func, G_TYPE_INT, NULL, NULL, hash_func, equal_func, NULL), _g_object_unref0 (self->priv->item_count), _tmp0_);
#line 1179 "TimedQueue.c"
	return self;
}


#line 212 "TimedQueue.vala"
HashTimedQueue* hash_timed_queue_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, guint hold_msec, DequeuedCallback callback, void* callback_target, GHashFunc hash_func, GEqualFunc equal_func, gint priority) {
#line 212 "TimedQueue.vala"
	return hash_timed_queue_construct (TYPE_HASH_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, hash_func, equal_func, priority);
#line 1188 "TimedQueue.c"
}


#line 219 "TimedQueue.vala"
static void hash_timed_queue_real_notify_dequeued (TimedQueue* base, gconstpointer item) {
#line 1194 "TimedQueue.c"
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (base);
#line 220 "TimedQueue.vala"
	hash_timed_queue_removed (self, item);
#line 222 "TimedQueue.vala"
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->notify_dequeued (TIMED_QUEUE (self), item);
#line 1201 "TimedQueue.c"
}


#line 225 "TimedQueue.vala"
static void hash_timed_queue_real_clear (TimedQueue* base) {
#line 1207 "TimedQueue.c"
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (base);
#line 226 "TimedQueue.vala"
	gee_abstract_map_clear (GEE_ABSTRACT_MAP (self->priv->item_count));
#line 228 "TimedQueue.vala"
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->clear (TIMED_QUEUE (self));
#line 1214 "TimedQueue.c"
}


#line 231 "TimedQueue.vala"
static gboolean hash_timed_queue_real_contains (TimedQueue* base, gconstpointer item) {
#line 1220 "TimedQueue.c"
	HashTimedQueue * self;
	gboolean result = FALSE;
	self = HASH_TIMED_QUEUE (base);
	result = gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item);
#line 232 "TimedQueue.vala"
	return result;
#line 1227 "TimedQueue.c"
}


#line 235 "TimedQueue.vala"
static gboolean hash_timed_queue_real_enqueue (TimedQueue* base, gconstpointer item) {
#line 1233 "TimedQueue.c"
	HashTimedQueue * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	self = HASH_TIMED_QUEUE (base);
#line 236 "TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue (TIMED_QUEUE (self), item)) {
#line 1240 "TimedQueue.c"
		result = FALSE;
#line 237 "TimedQueue.vala"
		return result;
#line 1244 "TimedQueue.c"
	}
#line 239 "TimedQueue.vala"
	if (gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item)) {
#line 239 "TimedQueue.vala"
		_tmp0_ = GPOINTER_TO_INT (gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item)) + 1;
#line 1250 "TimedQueue.c"
	} else {
#line 239 "TimedQueue.vala"
		_tmp0_ = 1;
#line 1254 "TimedQueue.c"
	}
#line 239 "TimedQueue.vala"
	gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (_tmp0_));
#line 1258 "TimedQueue.c"
	result = TRUE;
#line 241 "TimedQueue.vala"
	return result;
#line 1262 "TimedQueue.c"
}


#line 244 "TimedQueue.vala"
static gboolean hash_timed_queue_real_enqueue_many (TimedQueue* base, GeeCollection* items) {
#line 1268 "TimedQueue.c"
	HashTimedQueue * self;
	gboolean result = FALSE;
	self = HASH_TIMED_QUEUE (base);
#line 244 "TimedQueue.vala"
	g_return_val_if_fail (GEE_IS_COLLECTION (items), FALSE);
#line 245 "TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue_many (TIMED_QUEUE (self), items)) {
#line 1276 "TimedQueue.c"
		result = FALSE;
#line 246 "TimedQueue.vala"
		return result;
#line 1280 "TimedQueue.c"
	}
	{
		GeeIterator* _item_it;
#line 248 "TimedQueue.vala"
		_item_it = gee_iterable_iterator (GEE_ITERABLE (items));
#line 248 "TimedQueue.vala"
		while (TRUE) {
#line 1288 "TimedQueue.c"
			gpointer item;
			gint _tmp0_ = 0;
#line 248 "TimedQueue.vala"
			if (!gee_iterator_next (_item_it)) {
#line 248 "TimedQueue.vala"
				break;
#line 1295 "TimedQueue.c"
			}
#line 248 "TimedQueue.vala"
			item = gee_iterator_get (_item_it);
#line 249 "TimedQueue.vala"
			if (gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item)) {
#line 249 "TimedQueue.vala"
				_tmp0_ = GPOINTER_TO_INT (gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item)) + 1;
#line 1303 "TimedQueue.c"
			} else {
#line 249 "TimedQueue.vala"
				_tmp0_ = 1;
#line 1307 "TimedQueue.c"
			}
#line 249 "TimedQueue.vala"
			gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (_tmp0_));
#line 1311 "TimedQueue.c"
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		_g_object_unref0 (_item_it);
	}
	result = TRUE;
#line 251 "TimedQueue.vala"
	return result;
#line 1319 "TimedQueue.c"
}


#line 254 "TimedQueue.vala"
static gboolean hash_timed_queue_real_remove_first (TimedQueue* base, gconstpointer item) {
#line 1325 "TimedQueue.c"
	HashTimedQueue * self;
	gboolean result = FALSE;
	self = HASH_TIMED_QUEUE (base);
#line 255 "TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->remove_first (TIMED_QUEUE (self), item)) {
#line 1331 "TimedQueue.c"
		result = FALSE;
#line 256 "TimedQueue.vala"
		return result;
#line 1335 "TimedQueue.c"
	}
#line 258 "TimedQueue.vala"
	hash_timed_queue_removed (self, item);
#line 1339 "TimedQueue.c"
	result = TRUE;
#line 260 "TimedQueue.vala"
	return result;
#line 1343 "TimedQueue.c"
}


#line 263 "TimedQueue.vala"
static void hash_timed_queue_removed (HashTimedQueue* self, gconstpointer item) {
#line 1349 "TimedQueue.c"
	gint count;
#line 263 "TimedQueue.vala"
	g_return_if_fail (IS_HASH_TIMED_QUEUE (self));
#line 264 "TimedQueue.vala"
	g_assert (gee_abstract_map_has_key (GEE_ABSTRACT_MAP (self->priv->item_count), item));
#line 266 "TimedQueue.vala"
	count = GPOINTER_TO_INT (gee_abstract_map_get (GEE_ABSTRACT_MAP (self->priv->item_count), item));
#line 267 "TimedQueue.vala"
	g_assert (count > 0);
#line 269 "TimedQueue.vala"
	if ((count = count - 1) == 0) {
#line 270 "TimedQueue.vala"
		gee_abstract_map_unset (GEE_ABSTRACT_MAP (self->priv->item_count), item, NULL);
#line 1363 "TimedQueue.c"
	} else {
#line 272 "TimedQueue.vala"
		gee_abstract_map_set (GEE_ABSTRACT_MAP (self->priv->item_count), item, GINT_TO_POINTER (count));
#line 1367 "TimedQueue.c"
	}
}


static void hash_timed_queue_class_init (HashTimedQueueClass * klass) {
	hash_timed_queue_parent_class = g_type_class_peek_parent (klass);
	TIMED_QUEUE_CLASS (klass)->finalize = hash_timed_queue_finalize;
	g_type_class_add_private (klass, sizeof (HashTimedQueuePrivate));
	TIMED_QUEUE_CLASS (klass)->notify_dequeued = hash_timed_queue_real_notify_dequeued;
	TIMED_QUEUE_CLASS (klass)->clear = hash_timed_queue_real_clear;
	TIMED_QUEUE_CLASS (klass)->contains = hash_timed_queue_real_contains;
	TIMED_QUEUE_CLASS (klass)->enqueue = hash_timed_queue_real_enqueue;
	TIMED_QUEUE_CLASS (klass)->enqueue_many = hash_timed_queue_real_enqueue_many;
	TIMED_QUEUE_CLASS (klass)->remove_first = hash_timed_queue_real_remove_first;
}


static void hash_timed_queue_instance_init (HashTimedQueue * self) {
	self->priv = HASH_TIMED_QUEUE_GET_PRIVATE (self);
}


static void hash_timed_queue_finalize (TimedQueue* obj) {
	HashTimedQueue * self;
	self = HASH_TIMED_QUEUE (obj);
	_g_object_unref0 (self->priv->item_count);
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->finalize (obj);
}


GType hash_timed_queue_get_type (void) {
	static volatile gsize hash_timed_queue_type_id__volatile = 0;
	if (g_once_init_enter (&hash_timed_queue_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HashTimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) hash_timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HashTimedQueue), 0, (GInstanceInitFunc) hash_timed_queue_instance_init, NULL };
		GType hash_timed_queue_type_id;
		hash_timed_queue_type_id = g_type_register_static (TYPE_TIMED_QUEUE, "HashTimedQueue", &g_define_type_info, 0);
		g_once_init_leave (&hash_timed_queue_type_id__volatile, hash_timed_queue_type_id);
	}
	return hash_timed_queue_type_id__volatile;
}




